Package: DNAshapeR
Type: Package
Title: High-throughput prediction of DNA shape features
Version: 1.41.0
Date: 2018-06-08
Author: Tsu-Pei Chiu and Federico Comoglio
Maintainer: Tsu-Pei Chiu <tsupeich@usc.edu>
Description: DNAhapeR is an R/BioConductor package for ultra-fast,
        high-throughput predictions of DNA shape features. The package
        allows to predict, visualize and encode DNA shape features for
        statistical learning.
License: GPL-2
biocViews: StructuralPrediction, DNA3DStructure, Software
Depends: R (>= 3.4), GenomicRanges
Imports: Rcpp (>= 0.12.1), Biostrings, fields
LinkingTo: Rcpp
NeedsCompilation: yes
Packaged: 2026-05-30 08:48:15 UTC; root
VignetteBuilder: knitr
Suggests: AnnotationHub, knitr, rmarkdown, testthat,
        BSgenome.Scerevisiae.UCSC.sacCer3, BSgenome.Hsapiens.UCSC.hg19,
        caret
RoxygenNote: 6.0.1
Config/pak/sysreqs: zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:43:02 UTC
RemoteUrl: https://github.com/bioc/DNAshapeR
RemoteRef: HEAD
RemoteSha: d2b3810232e86916f6064c68fdd85c319c52d566
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-30 08:54:14 UTC; unix
