DNAshapeR
----

DNAshapeR is a R/BioConductor package for DNA shape prediction and feature encoding.

The latest (devel) version of DNAshapeR requires [R >3.2.2](https://cran.r-project.org) and can be installed from this GitHub repository with:

```R
 library(devtools)
 install_github(repo = "TsuPeiChiu/DNAshapeR", build_vignettes = TRUE)
```

For more information, package vignette and documentation, please visit [http://tsupeichiu.github.io/DNAshapeR/](http://tsupeichiu.github.io/DNAshapeR/)
