# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

getDNAShape <- function(fastaFilePath, shapeType) {
    .Call('_DNAshapeR_getDNAShape', PACKAGE = 'DNAshapeR', fastaFilePath, shapeType)
}
